<?php

/**
* Controller that is responsible for handling project events related requests
*
* @version 1.0
* @author Marcos Saiz <marcos.saiz@gmail.com>
* @adapted from Reece calendar <http://reececalendar.sourceforge.net/>.
* Acknowledgements at the bottom.
*/

class EventController extends ApplicationController {

	/**
	* Construct the EventController
	*
	* @access public
	* @param void
	* @return EventController
	*/
	function __construct() {
		parent::__construct();
		prepare_company_website_controller($this, 'website');	
		// Reece calendar initialization
		//define('CAL_SECURITY_BIT',1);
		$_SESSION['cal_loginfailed'] = 0;
		$_SESSION['cal_user'] = logged_user()->getUsername();
		$_SESSION['cal_userid'] = logged_user()->getId() ;
		cal_add_to_links('c=event');
		cal_load_permissions();
	} // __construct
     	
	/**
	* Show events index page (list recent events)
	*
	* @param void
	* @return null
	*/
	function index() {
		//auth check in cal_query_get_eventlist		
		if( (!(logged_user()->isAdministrator())) && ((active_project() && !(logged_user()->isProjectUser(active_project()))))){	    	
			flash_error(lang('no access permissions'));
			$this->redirectTo('dashboard');
			return ;
	    }		
		ajx_set_no_toolbar(true);
		ajx_replace(true);
		
		if(!active_project() || ProjectEvent::canAdd(logged_user(),active_project())) {
					 add_page_action(lang('add event'), get_url('event','add',
					 		array("year"=> date("Y"),
					 			 "month" => date("m"),
					 			 "day" => date('d')
					 			 )
					 		), 'ico-event'
					 );// cal_getlink('index.php?'.$today) );
				 }

				 
		$year = isset($_GET['year'])?$_GET['year']:$_SESSION['cal_year'];
		$month = isset($_GET['month'])?$_GET['month']:$_SESSION['cal_month'];
		$day = isset($_GET['day'])?$_GET['day']:$_SESSION['cal_day'];
		  
		  $pm = $month - 1;
		  $py = $year;
		  if($pm==0){
		  	$pm = 12;
			$py--;
		  }
		  $nm = $month + 1;
		  $ny = $year;
		  if($nm==13){
		  	$nm = 1;
			$ny++;
		  } 
		add_page_action(lang('today'), get_url('event','index',
		 		array("year"=> date("Y"),
		 			 "month" => date("m"),
		 			 "day" => date('d')
		 			 )
		 		), 'ico-today'
		 );// cal_getlink('index.php?'.$today) );
		 add_page_action(lang('week'), get_url('event','viewweek',
		 		array("year"=>  $year,
		 			 "month" => $month,
		 			 "day" => $day
		 			 )
		 		), 'ico-calendar-week'
		 );
		 add_page_action(lang('previous month'), get_url('event','index',
		 		array("year"=> $py,
		 			 "month" => $pm
		 			 )
		 		), 'ico-prevmonth'
		 );// add_page_action(lang('previous month'), cal_getlink("index.php?year=$py&month=$pm"));
		 add_page_action(lang('next month'), get_url('event','index',
		 		array("year"=> $ny,
		 			 "month" => $nm
		 			 )
		 		), 'ico-nextmonth'
		 );
		 
		 ajx_replace(true);
		 
	    $tag = active_tag();
		tpl_assign('tags',$tag);
		$this->setTemplate('calendar');
	}
	
	function getData($event_data){
		// get the day
			$day = array_var($event_data, 'start_day');
       		$month = array_var($event_data, 'start_month');
       		$year = array_var($event_data, 'start_year');
       		
       		$hour = array_var($event_data, 'hour')==0?12:array_var($event_data, 'hour');
       		$minute = array_var($event_data, 'minute');       		
			if(array_var($event_data, 'pm') == 1) $hour += 12;
			if (array_var($event_data, 'type_id') == 2 && $hour==24) $hour=23;
			// make sure the date is actually valid
			$redotime = mktime(0,0,1,$month, $day, $year);
			$day = date("d",$redotime);
			$month = date("m",$redotime);
			$year = date("Y",$redotime);
			
			
			// repeat defaults
			$repeat_d = 0;
			$repeat_m = 0;
			$repeat_y = 0;
			$repeat_h = 0;
			$rend = "0000-00-00";		
			// get the options
			$forever = 0;
			$jump = array_var($event_data,'occurance_jump');
			
			if(array_var($event_data,'repeat_option')==1) $forever = 1;
			elseif(array_var($event_data,'repeat_option')==2) $rnum = array_var($event_data,'repeat_num');
			elseif(array_var($event_data,'repeat_option')==3) $rend = array_var($event_data,'repeat_end');
			// verify the options above are valid
			if(isset($rnum) && $rnum !=""){
				if(!is_numeric($rnum) || $rnum < 1 || $rnum > 1000) {throw new Exception(CAL_EVENT_COUNT_ERROR);}
			}else $rnum = 0;
			if($jump !=""){
				if(!is_numeric($jump) || $jump<1 || $jump>1000) {throw new Exception(CAL_REPEAT_EVERY_ERROR);}
			}else $jump = 1;
			if(isset($rend) && $rend!=null && $rend!=""){
					$endarray = explode("-",$rend);
					if(count($endarray)!=3) {throw new Exception(CAL_ENDING_DATE_ERROR);}
					foreach($endarray as $v){ if(!is_numeric($v)) {throw new Exception(CAL_ENDING_DATE_ERROR);}}
					$rend = date("Y-m-d",mktime(0,0,1,$endarray[1], $endarray[2], $endarray[0]));
			}
			
		
		    // check for repeating options
			// 1=repeat once, 2=repeat daily, 3=weekly, 4=monthy, 5=yearly, 6=holiday repeating
			$oend = null;
			switch(array_var($event_data,'occurance')){
				case "2":
					$repeat_d = $jump;
					if(isset($forever) && $forever==1) $oend = null;
					else $oend = $rend;
					break;
				case "3":
					$repeat_d = 7*$jump;
					if(isset($forever) &&$forever==1) $oend = null;
					else $oend = $rend;
					break;
				case "4":
					$repeat_m = $jump;
					if(isset($forever) &&$forever==1) $oend = null;
					else $oend = $rend;
					break;
				case "5":
					$repeat_y = $jump;
					if(isset($forever) && $forever==1) $oend = null;
					else $oend = $rend;
					break;
				case "6":
					$repeat_h = 1;
					if(array_var($event_data, 'cal_holiday_lastweek')) $repeat_h = 2;
					break;
			}
			$repeat_number = $rnum;
			
			
		 	// get duration
			$durationhour = array_var($event_data,'durationhour');
			$durationmin = array_var($event_data,'durationmin');

			// get event type:  2=full day, 3=time/duratin not specified, 4=time not specified
			$typeofevent = array_var($event_data,'type_id');
			if(!is_numeric($typeofevent) OR ($typeofevent!=1 AND $typeofevent!=2 AND $typeofevent!=3)) $typeofevent = 1;
					
						
			// calculate timestamp and durationstamp
			// By putting through mktime(), we don't have to check for sql injection here and ensure the date is valid at the same time.
			$timestamp = date('Y-m-d H:i:s', mktime($hour+null,$minute+null,0,$month,$day,$year));
			$durationstamp = date('Y-m-d H:i:s', mktime($hour + $durationhour,$minute + $durationmin, 0,$month,$day,$year)); 
			
			// organize the data expected by the query function
			$data = array();
			$data['repeat_num'] = $rnum;
			$data['repeat_h'] = $repeat_h;
			$data['repeat_d'] = $repeat_d;
			$data['repeat_m'] = $repeat_m;
			$data['repeat_y'] = $repeat_y;
			$data['repeat_forever'] = $forever;
			$data['repeat_end'] =  $oend;
			$data['start'] = $timestamp;
			$data['subject'] =  array_var($event_data,'subject');
			$data['description'] =  array_var($event_data,'description');
			$data['type_id'] = $typeofevent;
			$data['duration'] = $durationstamp;
			
			return $data;
	}
	
	function add(){		
		if(! (ProjectEvent::canAdd(logged_user(), active_or_personal_project()))){	    	
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return ;
	    }
	    $this->setTemplate('event');
		$event = new ProjectEvent();		
		$event_data = array_var($_POST, 'event');
		$event_subject = array_var($_GET, 'subject'); //if sent from pupup
		
		$month = isset($_GET['month'])?$_GET['month']:date('n');
		$day = isset($_GET['day'])?$_GET['day']:date('j');
		$year = isset($_GET['year'])?$_GET['year']:date('Y');
		
		$event->setStart(DateTimeValueLib::make(12,0,0,$month,$day,$year));		
		$event->setDuration(DateTimeValueLib::make(13,0,0,$month,$day,$year));
		
		
		if(!is_array($event_data)) {
			// if data sent from quickadd popup (via get) we se it, else default
			$hour = isset($_GET['hour'])?$_GET['hour']:date('i');
			if(!cal_option("hours_24")){
				if($hour >= 12){
					$pm = 1;
					$hour = $hour - 12;
				}else $pm = 0;
			}
			$event_data = array(
				'month' => isset($_GET['month'])?$_GET['month']:date('n'),
				'year' => isset($_GET['year'])?$_GET['year']:date('Y'),
				'day' => isset($_GET['day'])?$_GET['day']:date('n'),
				'hour' => $hour,
				'minute' => isset($_GET['minute'])?$_GET['minute']:date('i'),
				'pm' => $pm,
				'typeofevent' => isset($_GET['type_id'])?$_GET['type_id']:1,
				'subject' => $event_subject
			); // array
		} // if
		
		
		tpl_assign('event',$event);
		tpl_assign('event_data',$event_data);
		tpl_assign('active_projects',logged_user()->getActiveProjects());

		if (is_array(array_var($_POST, 'event'))) {
			try {
				$data = $this->getData($event_data);
				
				// run the query to set the event data 
				$projId = array_var($event_data,'project_id');      
				if($projId != '') {
					$project = Projects::findById($projId );
				}
				else {
			 		$project = active_or_personal_project();
				}
	        	$event->setProjectId($project->getId());
			    $event->setFromAttributes($data); 
			    
			    if(!logged_user()->isMemberOfOwnerCompany()) $event->setIsPrivate(false);  
		
			    DB::beginWork();
	          	$event->save();
	          	$event->setTagsFromCSV(array_var($event_data, 'tags'));   
	            $event->save_properties(array_var($event_data,'event'));
			    
			    $object_controller = new ObjectController();
			    $object_controller->link_to_new_object($event);
	         	DB::commit();
	          	ApplicationLogs::createLog($event, $project, ApplicationLogs::ACTION_ADD);
	          	
	          	flash_success(lang('success add event', $event->getObjectName()));
	          	
	          	if (array_var($_POST, 'popup', false)) {
					ajx_current("reload");
	          	} else {
	          		ajx_current("back");
	          	}
	        } catch(Exception $e) {
	          	DB::rollback();
				flash_error($e->getMessage());
				ajx_current("empty");
	        } // try
		    
		}
	}
	

	function delete(){
		//check auth
		$event = ProjectEvents::findById(get_id());
	    if(!$event->canDelete(logged_user())){	    	
			flash_error(lang('no access permissions'));
			$this->redirectTo('event');
			return ;
	    }	
		$this->setTemplate('viewdate');
		$tag = active_tag();
		tpl_assign('tags',$tag);
		//tpl_assign('cal_action','calendar');
		//tpl_assign('action','calendar');
		try {
			/* @event ProjectEvent */
			
			DB::beginWork();
			$event->delete();
			ApplicationLogs::createLog($event, $event->getProject(), ApplicationLogs::ACTION_DELETE);
			DB::commit();

			flash_success(lang('success delete event', $event->getSubject()));
			ajx_current("back");
		} catch(Exception $e) {
			DB::rollback();
			flash_error(lang('error delete event'));
			ajx_current("empty");
		} // try
	}
	
	function viewdate(){
		$tag = active_tag();
		tpl_assign('tags',$tag);	
		tpl_assign('cal_action','viewdate');
		ajx_set_no_toolbar(true);
		
		$day = isset($_GET['day'])?$_GET['day']:date('j');
		$month = isset($_GET['month'])?$_GET['month']:date('n');
	    $year = isset($_GET['year'])?$_GET['year']:date('Y');
		
		if(!active_project() || ProjectEvent::canAdd(logged_user(),active_project())) {
					 add_page_action(lang('add event'), get_url('event','add',
					 		array("year"=> $year,
					 			 "month" =>  $month,
					 			 "day" => $day
					 			 )
					 		), 'ico-event'
					 );// cal_getlink('index.php?'.$today) );
				 }
		
		
		add_page_action(lang('month'), get_url('event','index',
		 		array("year"=> $year,
		 			 "month" => $month
		 			 )
		 		), 'ico-calendar-month'
		);
		
		add_page_action(lang('week'), get_url('event','viewweek',
		 		array("year"=> $year,
		 			 "month" => $month,
		 			 "day" => $day
		 			 )
		 		), 'ico-calendar-week'
		);
		 
		 
		$this->setTemplate('viewdate');		
	}
	
	function viewweek(){
		$tag = active_tag();
		tpl_assign('tags',$tag);	
		tpl_assign('cal_action','viewdate');
		ajx_set_no_toolbar(true);
		
				
		$day = isset($_GET['day'])?$_GET['day']:date('j');
		$month = isset($_GET['month'])?$_GET['month']:date('n');
	    $year = isset($_GET['year'])?$_GET['year']:date('Y');
		add_page_action(lang('month'), get_url('event','index',
		 		array("year"=> $year,
		 			 "month" => $month
		 			 )
		 		), 'ico-calendar-month'
		);
		
		$next_week = getDate(strtotime('+1 week',mktime(0, 0, 0, $month, $day , $year)));
        $prev_week = getDate(strtotime('-1 week',mktime(0, 0, 0, $month, $day , $year)));
         
		add_page_action(lang('next week'), get_url('event','viewweek',
		 		array("day"=> $next_week["mday"],
		 			 "year"=> $next_week["year"],
		 			 "month" => $next_week["mon"]
		 			 )
		 		), 'ico-nextmonth'
		);
		
		add_page_action(lang('previous week'), get_url('event','viewweek',
		 		array("day"=> $prev_week["mday"],
		 			 "year"=> $prev_week["year"],
		 			 "month" => $prev_week["mon"]
		 			 )
		 		), 'ico-prevmonth'
		);
		 
		 
		$this->setTemplate('viewweek');		
	}
	
	
	
	function viewevent(){
		//check auth
		$this->addHelper('textile');
		ajx_set_no_toolbar(true);
	    $event = ProjectEvents::findById(get_id());
	    if(!$event->canView(logged_user())){	    	
			flash_error(lang('no access permissions'));
			$this->redirectTo('event');
			return ;
	    }
		$this->setTemplate('viewevent');
		$tag = active_tag();
		tpl_assign('tags',$tag);	
		tpl_assign('event',$event);
		tpl_assign('cal_action','viewevent');	
		tpl_assign('view', array_var($_GET, 'view','month'));	
		tpl_assign('active_projects',logged_user()->getActiveProjects());
		ajx_extra_data(array("title" => $event->getSubject(), 'icon'=>'ico-calendar'));
	}
	/*function search(){
		$this->setTemplate('search');		
	}*/
	function cal_error($text){
		$output = "<center><span class='failure'>$text</span></center><br>";
		return $output;
	}
	
	
	
		
	function edit() {
		$this->setTemplate('event');
		$event = ProjectEvents::findById(get_id());
		if(!$event->canEdit(logged_user())){	    	
			flash_error(lang('no access permissions'));
			ajx_current("empty");
			return ;
	    }
	    
		tpl_assign('active_projects',logged_user()->getActiveProjects());
	    
		$event_data = array_var($_POST, 'event');
		if(!is_array($event_data)) {
			$tag_names = $event->getTagNames();
			$setlastweek = false;
			$rsel1=false;$rsel2=false; $rsel3=false;
			$forever= $event->getRepeatForever();
			$occ = 1;
			if($event->getRepeatD()  > 0){ $occ = 2; $rjump = $event->getRepeatD();}
			if($event->getRepeatD() > 0 AND $event->getRepeatD()%7==0){ $occ = 3; $rjump = $event->getRepeatD()/7;}
			if($event->getRepeatM() > 0){ $occ = 4; $rjump = $event->getRepeatM();}
			if($event->getRepeatY() > 0){ $occ = 5; $rjump = $event->getRepeatY();}
			if($event->getRepeatH() > 0){ $occ = 6;}
			if($event->getRepeatH()==2){ $setlastweek = true;}
			if($event->getRepeatEnd()) { $rend = $event->getRepeatEnd()->format('Y-m-d');	}
			if($event->getRepeatNum() > 0) $rnum = $event->getRepeatNum();
			if(!isset($rjump) || !is_numeric($rjump)) $rjump = 1;
			// decide which repeat type it is
			if($forever) $rsel1 = true; //forever
			else if(isset($rnum) AND $rnum>0) $rsel2 = true; //repeat n-times
			else if(isset($rend) AND $rend!="") $rsel3 = true; //repeat until
			
			if(isset($rend) AND $rend=="9999-00-00") $rend = "";
			// organize the time and date data for the html select drop downs.
			$thetime = $event->getStart()->getTimestamp();
			$durtime = $event->getDuration()->getTimestamp() - $thetime;
			$hour = date('G', $thetime);
			// format time to 24-hour or 12-hour clock.
			if(!cal_option("hours_24")){
				if($hour >= 12){
					$pm = 1;
					$hour = $hour - 12;
				}else $pm = 0;
			}
			
						
			$event_data = array(
	          'subject'        => $event->getSubject(),
	          'description'        => $event->getDescription(),
	          'name'    => $event->getCreatedById(),
	          'username' => $event->getCreatedById(),
	          'typeofevent'  => $event->getTypeId(),
	          'forever'  => $event->getRepeatForever(),
	          'usetimeandduration'  => ($event->getTypeId())==3?0:1,
	          'occ' => $occ,
	          'rjump' => $rjump,
	          'setlastweek' => $setlastweek,
	          'rend' => isset($rend)?$rend:NULL,
	          'rnum' => isset($rnum)?$rnum:NULL,
	          'rsel1' => $rsel1,
	          'rsel2' => $rsel2,
	          'rsel3' => $rsel3,
	          'thetime' => $event->getStart()->getTimestamp(),
			  'hour' => $hour,
			  'minute' => date('i', $thetime),
			  'month' => date('n', $thetime),
			  'year' => date('Y', $thetime),
			  'day' => date('j', $thetime),
			  'durtime' => ($event->getDuration()->getTimestamp() - $thetime),
			  'durmin' => ($durtime / 60) % 60,
			  'durhr'  => ($durtime / 3600) % 24,
			  'durday' => floor($durtime / 86400),
			  'pm' => $pm,
	          'tags' => is_array($tag_names) ? implode(', ', $tag_names) : ''
				); // array
			} // if
	
			tpl_assign('event_data', $event_data);
			tpl_assign('event', $event);
	
			if(is_array(array_var($_POST, 'event'))) {
				try {
					$data = $this->getData($event_data);
					// run the query to set the event data 
					$projId = array_var($event_data,'project_id');                    
				
					if($projId != '') {
						$project = Projects::findById($projId );
					}
					else {
						$project = $event->getProject();
					} 
		        	$event->setProjectId($project->getId());
				    $event->setFromAttributes($data); 
				    
				    if(!logged_user()->isMemberOfOwnerCompany()) $event->setIsPrivate(false);  
				
		          
		          	DB::beginWork();
		         	$event->save();
		         	$event->setTagsFromCSV(array_var($event_data, 'tags')); 
				 	$event->save_properties(array_var($event_data,'event'));
		          	ApplicationLogs::createLog($event, $project, ApplicationLogs::ACTION_ADD);
		          	DB::commit();
		          
		          	flash_success(lang('success edit event', $event->getObjectName()));
					ajx_current("back");						          
		          
		        } catch(Exception $e) {
		        	DB::rollback();
					flash_error($e->getMessage());
					ajx_current("empty");		          
		          //tpl_assign('error', $e);
		        } // try
			} // if
	} // edit
	

} // EventController

/***************************************************************************
 *           Parts of the code for this class were extracted from
 *                            -------------------
 *   begin                : Saturday, Feb 13, 2001
 *   copyright            : (C) 2001 The phpBB Group
 *   email                : support@phpbb.com
 *
 *   $Id: EventController.class.php,v 1.44 2008/10/02 22:33:30 nicomede Exp $
 *
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 ***************************************************************************/
/*
	Code is from:
	Copyright (c) Reece Pegues
	sitetheory.com

    Reece PHP Calendar is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or 
	any later version if you wish.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation Inc, 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
*/
?>